import uuid

from datetime import date, datetime
from decimal import Decimal
from typing import List, Union

from pydantic import BaseModel, validator

from . import to_camel
from .user_link import UserLink


class Entries(BaseModel):
    id_: uuid.UUID
    date_: date
    url: List[str]
    type_: str
    posted: bool
    narration: str
    debit_names: List[str]
    credit_names: List[str]
    amount: Decimal
    creation_date: datetime
    last_edit_date: datetime
    user: UserLink

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {
            date: lambda v: v.strftime("%d-%b-%Y"),
            datetime: lambda v: v.strftime("%d-%b-%Y %H:%M"),
        }

    @validator("date_", pre=True)
    def parse_date(cls, value: Union[date, str]) -> date:
        if isinstance(value, date):
            return value
        return datetime.strptime(value, "%d-%b-%Y").date()

    @validator("creation_date", pre=True)
    def parse_creation_date(cls, value: Union[datetime, str]) -> datetime:
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")

    @validator("last_edit_date", pre=True)
    def parse_last_edit_date(cls, value: Union[datetime, str]) -> datetime:
        if isinstance(value, datetime):
            return value
        return datetime.strptime(value, "%d-%b-%Y %H:%M")


class Report(BaseModel):
    counts: int
    report: List[Entries]

    class Config:
        anystr_strip_whitespace = True
        alias_generator = to_camel
        json_encoders = {
            date: lambda v: v.strftime("%d-%b-%Y"),
            datetime: lambda v: v.strftime("%d-%b-%Y %H:%M"),
        }
