import { CommonModule } from '@angular/common';
import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';

import { AuthGuard } from '../auth/auth-guard.service';
import { ProductGroupListResolver } from '../product-group/product-group-list-resolver.service';

import { ProductDetailComponent } from './product-detail/product-detail.component';
import { ProductListResolver } from './product-list-resolver.service';
import { ProductListComponent } from './product-list/product-list.component';
import { ProductResolver } from './product-resolver.service';

const productRoutes: Routes = [
  {
    path: '',
    component: ProductListComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Products',
    },
    resolve: {
      list: ProductListResolver,
    },
  },
  {
    path: 'new',
    component: ProductDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Products',
    },
    resolve: {
      item: ProductResolver,
      productGroups: ProductGroupListResolver,
    },
  },
  {
    path: ':id',
    component: ProductDetailComponent,
    canActivate: [AuthGuard],
    data: {
      permission: 'Products',
    },
    resolve: {
      item: ProductResolver,
      productGroups: ProductGroupListResolver,
    },
  },
];

@NgModule({
  imports: [CommonModule, RouterModule.forChild(productRoutes)],
  exports: [RouterModule],
  providers: [ProductListResolver, ProductResolver],
})
export class ProductRoutingModule {}
